import {
  Metadata,
  Http,
  IEventParam,
  Loader,
  I18n,
} from '@zeedhi/core';
import {
  Select,
  DateRange,
  Form,
  IterableComponentRender,
  Text,
} from '@zeedhi/common';
import {} from '@zeedhi/teknisa-components-common';

export class PlanningWithdrawalController {
  private WaybillLaunchController = Loader.getInstance('WaybillLaunchController');

  private Utilities = Loader.getInstance('Utilities');

  public mainPageVisibility: boolean = true;

  public cardDetailsVisibility: boolean = false;

  public idSelected: string = '';

  public menuDate: string = '';

  public withdrawalDate: string = '';

  public paramsForProductsQuery: Record<string, string> = {
    CDPRATO: '',
    NRORG: '',
    DTRETIRADA: '',
    NRCARDAPIO: '',
    NRRETIRADA: '',
    CDTIPOPRAT: '',
    CDFILIAL: '',
    CDFILAUXPRAT: '',
  };

  public waybillPaginationVisibility: boolean = false;

  public isExpanded: boolean = true;

  public planningWithdrawalTitleVisibility: boolean = false;

  public nrlancestq: any;

  public mainCollapseCardName = '<h2>Retirada do Planejamento</h2>';

  public unitParams: any = {
    VALIDAR_ESTOQUE: '',
    DTRETIRADA: '',
  };

  public pending: string = 'PENDENTE';

  public evolved: string = 'EVOLUÍDO';

  public addWaybillButtonVisibility: boolean = false;

  public unitSelected: any;

  public getInitialData() {
    Http.post('/getInitialData').then((response) => {
      const data = response.data.dataset.initialData;
      localStorage.setItem('NRORG', data.NRORG);
      localStorage.setItem('LOGGED_BRANCH', data.LOGGED_BRANCH);
      this.filterUnitsByOperator(localStorage.getItem('NRORG'));
    });
    this.getCdNvProduto();
  }

  public getCdNvProduto() {
    Http.post('/getCdNvProduto').then((response) => {
      localStorage.setItem('CDNVPRODUTO', response.data.dataset.cdNvProduto[0].CDNVPRODUTO);
    });
  }

  public filterUnitsByOperator(nrorg: any) {
    const unitSelect = Metadata.getInstance<Select>('unitsForWithdrawal');
    const conditions = {
      NRORG: nrorg,
      CDOPERADOR: localStorage.getItem('CDOPERADOR'),
    };

    unitSelect.datasource.filter = conditions;
    unitSelect.datasource.get();
    unitSelect.value = localStorage.getItem('LOGGED_BRANCH');
  }

  public filterServicesByUnit() {
    const unitSelectValue = Metadata.getInstance<Select>('unitsForWithdrawal').value;
    const servicesSelect = Metadata.getInstance<Select>('unitServices');

    if (unitSelectValue !== null) {
      const conditions = {
        CDFILIAL: unitSelectValue.CDFILIAL,
      };
      servicesSelect.datasource.filter = conditions;
      servicesSelect.datasource.get();
    } else {
      servicesSelect.datasource.clearFilter();
    }

    if (unitSelectValue !== null) {
      this.loadWithdrawalDates();
    }
  }

  public loadWithdrawalDates() {
    const unitSelectValue = Metadata.getInstance<Select>('unitsForWithdrawal').value;
    const withdrawalDate = Metadata.getInstance<DateRange>('dateRangeForWithdrawal');
    const menuDate = Metadata.getInstance<DateRange>('dateRangeForMenu');
    const conditions = {
      CDFILIAL: unitSelectValue,
    };

    Http.post('/planningWithdrawalData', conditions).then((response) => {
      const params = response.data.dataset.planningWithdrawalData;
      withdrawalDate.value = [params[0].DTPROCESSA.substring(0, 10), params[0].DTPROCESSA.substring(0, 10)];
      menuDate.value = [params[0].DTPROCESSA.substring(0, 10), params[0].DTPROCESSA.substring(0, 10)];
    });
  }

  public async clickCard({ component }: any) {
    this.mainPageVisibility = false;
    this.cardDetailsVisibility = !this.mainPageVisibility;
    this.addWaybillButtonVisibility = false;

    const params = component.name.split('-');
    const unitSelect = Metadata.getInstance<Select>('unitsForWithdrawal');
    const waybills = await Metadata.getInstance<IterableComponentRender>('detailCardWaybills');
    const dishesTypes = await Metadata.getInstance<IterableComponentRender>('iterableCollapseCardDishesTypes');
    const products = await Metadata.getInstance<IterableComponentRender>('iterableProducts');
    const conditionsWaybill = {
      CDFILIAL: unitSelect.value.CDFILIAL,
      CDSERVICO: params[0],
      DTCARDINI: params[2].substring(0, 10),
      DTCARDFIN: params[2].substring(0, 10),
      DTRETINI: params[1].substring(0, 10),
      DTRETFIN: params[1].substring(0, 10),
    };
    waybills.datasource.filter = conditionsWaybill;
    waybills.datasource.get().then((response) => {
      this.nrlancestq = response[0].NRLANCESTQ;
      this.unitParams.DTRETIRADA = response[0].DTLANCESTQ.substring(0, 10);
      this.paramsForProductsQuery.NRCARDAPIO = response[0].NRCARDAPIO;
      this.paramsForProductsQuery.DTRETIRADA = response[0].DTLANCESTQ.substring(0, 10);
      this.paramsForProductsQuery.NRORG = response[0].NRORG;
      this.paramsForProductsQuery.NRRETIRADA = response[0].NRRETIRADA;
      this.paramsForProductsQuery.CDFILIAL = response[0].CDFILIMOVI;
      if (response[0].IDRETIPRAT === 'S') {
        this.WaybillLaunchController.dishesTypesTitleVisibility = true;
        this.WaybillLaunchController.dishesTypesPaginationVisibility = true;
        this.WaybillLaunchController.productsPaginationInDishesVisibility = true;
        const conditionDishesTypes = {
          CDFILIAL: response[0].CDFILIMOVI,
          NRRETIRADA: response[0].NRRETIRADA,
          NRCARDAPIO: response[0].NRCARDAPIO,
        };

        dishesTypes.datasource.filter = conditionDishesTypes;
        dishesTypes.datasource.get().then((item) => {
          this.paramsForProductsQuery.CDPRATO = item[0].CDPRATO;
          this.paramsForProductsQuery.CDTIPOPRAT = item[0].CDTIPOPRAT;
          this.paramsForProductsQuery.CDFILAUXPRAT = item[0].CDFILAUXPRAT;
          // if (dishesTypes.datasource.totalValue > dishesTypes.datasource.limit) {
          //   this.WaybillLaunchController.dishesTypesPaginationVisibility = false;
          // } else {
          //   this.WaybillLaunchController.dishesTypesPaginationVisibility = true;
          // }
        });
      } else if (response[0].IDRETIPRAT === 'N') {
        this.WaybillLaunchController.productsPaginationVisibility = true;
        this.WaybillLaunchController.productsTitleVisibility = true;
        products.datasource.filter = this.paramsForProductsQuery;
        products.datasource.get();
      }
    });
  }

  public returnToMainPage() {
    const waybills = Metadata.getInstance<IterableComponentRender>('detailCardWaybills');
    const dishesTypes = Metadata.getInstance<IterableComponentRender>('iterableCollapseCardDishesTypes');
    const products = Metadata.getInstance<IterableComponentRender>('iterableProducts');
    const withdrawalsCards = Metadata.getInstance<IterableComponentRender>('iterableWaybillCard');
    this.addWaybillButtonVisibility = true;
    waybills.datasource.clearFilter();
    dishesTypes.datasource.clearFilter();
    products.datasource.clearFilter();
    withdrawalsCards.reload();
    this.WaybillLaunchController.dishesTypesTitleVisibility = false;
    this.WaybillLaunchController.productsTitleVisibility = false;
    this.WaybillLaunchController.productsPaginationInDishesVisibility = false;
    this.WaybillLaunchController.productsPaginationVisibility = false;
    this.WaybillLaunchController.dishesTypesPaginationVisibility = false;
    this.cardDetailsVisibility = false;
    this.mainPageVisibility = !this.cardDetailsVisibility;
  }

  public loadWaybillData() {
    const form = Metadata.getInstance<Form>('planningWithdrawalForm');
    const validation = form.validate();
    if (validation) {
      this.waybillPaginationVisibility = true;
      this.isExpanded = false;
      this.planningWithdrawalTitleVisibility = true;

      const serviceSelected = Metadata.getInstance<Select>('unitServices');
      const dateMenu = Metadata.getInstance<DateRange>('dateRangeForMenu');
      const dateWithdrawal = Metadata.getInstance<DateRange>('dateRangeForWithdrawal');
      const unitSelect = Metadata.getInstance<Select>('unitsForWithdrawal');
      const waybillIterableCard = Metadata.getInstance<IterableComponentRender>('iterableWaybillCard');

      const conditions = {
        CDFILIAL: unitSelect.value.CDFILIAL,
        CDSERVICO: serviceSelected.value,
        DTCARDINI: dateMenu.value[0],
        DTCARDFIN: dateMenu.value[1],
        DTRETINI: dateWithdrawal.value[0],
        DTRETFIN: dateWithdrawal.value[1],
      };
      waybillIterableCard.datasource.filter = conditions;
      waybillIterableCard.datasource.get().then(() => {
        this.mainCollapseCardName = `${unitSelect.value.CDFILIAL} - ${unitSelect.value.NMFILIAL}`;
        this.loadUnitParameters();
        this.addWaybillButtonVisibility = true;
        this.unitSelected = unitSelect.value.CDFILIAL;
      });
    }
  }

  public loadUnitParameters() {
    const cdfilial = Metadata.getInstance<Select>('unitsForWithdrawal').value;
    const sendObject = {
      CDFILIAL: cdfilial,
    };
    Http.post('/loadUnitParameters', sendObject).then((response) => {
      const params = response.data.dataset.params[0];
      const idctrlestq = params.IDCTRLESTQ;
      const idutilalmox = params.IDUTILALMOX;
      const idutililote = params.IDUTILLOTE;
      params.VALIDAR_ESTOQUE = idctrlestq !== 'N' && (idutilalmox === 'S' || idutililote === 'S');
      this.unitParams.VALIDAR_ESTOQUE = JSON.parse(JSON.stringify(params.VALIDAR_ESTOQUE));
      this.unitParams = JSON.parse(JSON.stringify(params));
      const dayDtprocessa = this.unitParams.DTPROCESSA.substring(0, 2);
      const monthDtprocessa = this.unitParams.DTPROCESSA.substring(3, 5) - 1;
      const yearDtprocessa = this.unitParams.DTPROCESSA.substring(6, 10);
      const daysAfter = new Date(yearDtprocessa, monthDtprocessa, dayDtprocessa);
      const daysBefore = new Date(yearDtprocessa, monthDtprocessa, dayDtprocessa);
      daysBefore.setDate(daysBefore.getDate() - this.unitParams.NRDIASSAIDFIL);
      daysAfter.setDate(daysAfter.getDate() + this.unitParams.NRDIASSAIDPDFIL);
      this.unitParams.DIAS_ANTES_DTPROCESSA = daysBefore;
      this.unitParams.DIAS_APOS_DTPROCESSA = daysAfter;
    });
  }

  public updateStock({ component }: any) {
    const data = component.name.split('_');
    const sendObject = {
      CDFILIAL: data[1],
      CDFILIMOVI: Metadata.getInstance<Select>('unitsForWithdrawal').value.CDFILIAL,
      NRCARDAPIO: data[2],
      NRLANCESTQ: data[3],
    };

    Http.post('/updateStock', {
      requestType: 'Row',
      row: sendObject,
    }).then((response) => {
      if (response.data.error) {
        this.Utilities.failMessage(I18n.translate(response.data.error));
      } else {
        const stockUpdated = I18n.translate(this.Utilities.stockUpdated);
        this.Utilities.successMessage(stockUpdated);
        this.loadWaybillData();
      }
    });
  }

  public stylizeStatus({ component }: IEventParam<Text>) {
    const evolved = I18n.translate(this.evolved);
    const pending = I18n.translate(this.pending);
      if (component.text === 'N') {
          component.text = `<span  style='background-color:#dee2e6; color: #ff6700; font-size: 12px;'>${pending}</span>`;
      }
      if (component.text === 'S') {
          component.text = `<span  style='background-color: #dee2e6; color: green; font-size: 12px;'>${evolved}</span>`;
      }
      // if (component.text === 'Em andamento') {
      //     component.text = "<span  style='background-color: #dee2e6; color: #023e8a; font-size: 11px;'>EM ANDAMENTO</span>";
      // }
      // if (component.text === 'Cancelado') {
      //     component.text = "<span  style='background-color: #dee2e6; color: black; font-size: 10px;'>C A N C E L A D O</span>";
      // }
      // if (component.text === 'Atrasado') {
      //     component.text = "<span  style='background-color: #dee2e6; color: red; font-size: 10px;'>A T R A S A D O</span>";
      // }
  }

  public filteredDates({ component }: any) {
    const date = component.name.split('_');
    if (date[1].length > 9) {
      const filter = date[1].substring(0, 10);
      component.text = `<span>&nbsp${filter}</span>`;
    }
  }

  public setWithdrawalDate({ component }: any) {
    const withdrawalDate = Metadata.getInstance<DateRange>('dateRangeForWithdrawal');
    withdrawalDate.value = component.value;
  }
}
