import {
  IEventParam, I18n, Messages,
} from '@zeedhi/core';
import {
Number as NumberInput,
AlertService,
DialogService,
} from '@zeedhi/common';
import {} from '@zeedhi/teknisa-components-common';

export class Utilities {
  public stockUpdated: string = 'Estoque evoluído com sucesso.';

  public changeValueTo0({ component }: IEventParam<NumberInput>) {
    if (component.value === undefined) {
      component.value = 0;
    }
  }

  public failMessage(message: string) {
    AlertService.show({
      name: 'failedAlert',
      text: message,
      color: 'error',
      timeout: 5000,
      showDismiss: true,
      multiLine: true,
    });
  }

  public warnMessage(message: string) {
    AlertService.show({
      name: 'warnAlert',
      text: message,
      color: 'warning',
      timeout: 10000,
      showDismiss: true,
      multiLine: true,
    });
  }

  public successMessage(message: string) {
    AlertService.show({
      name: 'sucessAlert',
      text: message,
      color: 'success',
      showDismiss: true,
      timeout: 5000,
      multiLine: true,
    });
  }

  public showConfirmationDialog(titleString: string, textString: string, no: any, yes: any) {
    DialogService.show({
      name: 'buttons-dialog',
      type: 'warning',
      title: titleString,
      text: textString,
      buttons: [
        {
          name: 'no', component: 'ZdButton', label: 'Não', events: no, outline: true,
        },
        {
          name: 'yes', component: 'ZdButton', label: 'Sim', events: yes,
        },
      ],
    });
  }

  public getFormatLocale(): string {
    return (localStorage.getItem('LOGIN_LANGUAGE') !== 'en_us') ? 'DD/MM/YYYY' : 'MM/DD/YYYY';
  }

  public getLanguage() {
    return localStorage.getItem('LOGIN_LANGUAGE') || 'pt_br';
  }

  public formatDatesByLanguage({ component }: any) {
    const date = component.name.split('_');
    if (date[1].length > 9) {
      const filter = date[1].substring(1, 11);
      component.text = filter;
    }
    const condition = localStorage.getItem('LOGIN_LANGUAGE') === 'en_us' ? 'en' : 'pt';
    if (condition === 'en') {
      const dateDivided = component.text.split('/');
      const enDate = `<span>&nbsp${dateDivided[1]}/${dateDivided[0]}/${dateDivided[2]}</span>`;
      component.text = enDate;
    } else {
      component.text = `<span>&nbsp${component.text}</span>`;
    }
  }

  public showAlert(name: string, error: any, color = 'green', top = true, timeout = 3000) {
    const text = error?.response?.data;
    let msg;

    if (text) {
      msg = I18n.translate(`${text}`);
    } else {
      msg = I18n.translate(`RotinaEstoqueApp::${error}`);
    }

    const alertType = {
      name,
      color,
      text: msg,
      top,
      left: false,
      right: true,
      timeout,
      buttonsSlot: [
        {
          name: 'undo-button',
          component: 'ZdButton',
          icon: true,
          iconName: 'close',
          color: 'white',
          events: {
            click: () => { AlertService.hide(); },
          },
        },
      ],
    };
    AlertService.show(alertType);
  }

  public translateSplitter() {
    Messages.add({
      'pt-BR': {
        translation: {
          SPLITTER_TO: ' até ',
        },
      },
      'es-ES': {
        translation: {
          SPLITTER_TO: ' a ',
        },
      },
      'es-CL': {
        translation: {
          SPLITTER_TO: ' a ',
        },
      },
      'en-US': {
        translation: {
          SPLITTER_TO: ' to ',
        },
      },
    });
  }
}
